#include "OthelloApp.h"

#include <iostream>
#include <fstream>
#include <string>
#include "FKUT/Misc.h"

#include "AI_L1_Basic.h"
#include "AI_L2_Advanced.h"
#include "AI_L3_Extreme.h"
#include "AI_L4_Master.h"
#include "MyAI.h"


OthelloApp::OthelloApp(Fl_Window *argMainWin, fk_Window *argDrawWin)
	: fkut_AppBase(argMainWin, argDrawWin), tb(argDrawWin, &camera)
{
	// EBhEփV[o^
	drawWin->setScene(&scene);
	ai[ST_BLACK] = ai[ST_WHITE] = NULL;
}

OthelloApp::~OthelloApp(void)
{
	// EBhEV[̓o^
	drawWin->setScene(NULL);

	SAFE_DELETE(ai[ST_BLACK]);
	SAFE_DELETE(ai[ST_WHITE]);
}

void OthelloApp::init(void)
{
	// Ɩ̐ݒ
	lightModel.setShape(&light);
	lightModel.setMaterial(White);
	lightModel.getMaterial()->setDiffuse(1.0, 1.0, 1.0);
	lightModel.glMoveTo(0.0, 0.0, 0.0);
	lightModel.glFocus(-1.0, -1.0, -1.0);

	// zu4
	board.makeStone(fk_Vector( 5.0, 0.0,  5.0), ST_BLACK, &scene);
	board.makeStone(fk_Vector(-5.0, 0.0,  5.0), ST_WHITE, &scene);
	board.makeStone(fk_Vector(-5.0, 0.0, -5.0), ST_BLACK, &scene);
	board.makeStone(fk_Vector( 5.0, 0.0, -5.0), ST_WHITE, &scene);
	turn = ST_BLACK;

	// effBXvCXgɓo^
	scene.setBlendStatus(true);
	scene.setBGColor(0.5, 0.5, 0.5);
	scene.entryCamera(&camera);
	scene.entryModel(&lightModel);
	// NX̂Ńo֐oRŃGg[
	board.entry(&scene);

	// eLXgEl\p
	textSpr.initFont("C:/Windows/Fonts/Meiryo.ttc");
	textSpr.drawText("\̈");
	textSpr.entryFirst(drawWin, &scene, &camera);
	textSpr.text.setAlign(FK_ALIGN_LEFT);
	putMessage("Q[JnłB");

	// _̈ʒuƎpݒ
	camera.glMoveTo(0.0, 100.0, 100.0);
	camera.glFocus(0.0, 0.0, 0.0);
	camera.glUpvec(0.0, 1.0, 0.0);

	// [7/8]AINX̐
	int			choice[2];
	StoneColor	colors[2];
	choice[ST_BLACK] = atoi(fl_input("Is Black side?(0:HUM, 1-5:COM)", "0"));
	choice[ST_WHITE] = atoi(fl_input("Is White side?(0:HUM, 1-5:COM)", "0"));
	colors[ST_BLACK] = ST_BLACK;
	colors[ST_WHITE] = ST_WHITE;
	for(int i = 0; i < 2; ++i) {
		switch(choice[i]) {
		case 1:
			ai[i] = new AI_L1_Basic(colors[i]);
			break;
		case 2:
			ai[i] = new AI_L2_Advanced(colors[i]);
			break;
		case 3:
			ai[i] = new AI_L3_Extreme(colors[i]);
			break;
		case 4:
			ai[i] = new AI_L4_Master(colors[i]);
			break;
		case 5:
			ai[i] = new MyAI(colors[i]);
			break;
		default:
			ai[i] = NULL;
		}
	}

	return;
}

// NbNꏊ̍WlԂ(ՖʂɊւ鏈Board::checkCell())
fk_Vector OthelloApp::clickProc(void)
{
	fk_Plane	plane;
	fk_Vector	winPos, projPos;

	plane.setPosNormal(fk_Vector(0.0, 0.0, 0.0), fk_Vector(0.0, 1.0, 0.0));
	winPos = eveMan.getMousePosition();
	drawWin->getProjectPosition(winPos.x, winPos.y, &plane, &projPos);

	return projPos;
}

// t[̏
bool OthelloApp::update(void)
{
	// J̉]
	tb.update();
	// Cg̉]
	lightModel.glRotateWithVec(0.0, 0.0, 0.0, fk_Y, 0.01);

	// }EXꂽ炻̏ꏊɐ΂u
	if(turn != ST_DEFAULT) {
		if(eveMan.getMouseStatus(FK_MOUSE1) == FKUT_SW_DOWN) {
			procHand();
		} else if(ai[turn] != NULL && board.isAnimate() == false) {
			procHand();
		}
	}
	// t@C̏o(F2L[)Ɠ(F1L[)̃eXg
	if(eveMan.getSpecialKeyStatus(FK_F1) == FKUT_SW_DOWN) {
		if(loadState()) {
			putMessage("Load success.");
		} else {
			putMessage("Load failed.");
		}
	}
	if(eveMan.getSpecialKeyStatus(FK_F2) == FKUT_SW_DOWN) {
		if(saveState()) {
			putMessage("Save success.");
		} else {
			putMessage("Save failed.");
		}
	}

	board.update(turn);

	return true;
}

void OthelloApp::procHand(void)
{
	fk_Vector adjustPos, inputPos;

	// [7/8]΁AAIm̑Όɂ邱Ƃ\
	if(ai[turn] == NULL) {
		inputPos = clickProc();
	} else {
		cout << ((turn == ST_BLACK) ? "" : "") << "AI^[" << endl;
		if(ai[turn]->update(board.getInfo(), &inputPos) == false) {
			fl_alert("AI returned an unputtable hand!");
		}
	}

	// Ԓl̒lɉď𕪊
	int result = board.checkCell(inputPos, &adjustPos, turn, true);
	CellPosition pos = BoardInfo::calcID(adjustPos);
	switch(result) {
	case CR_SUCCESS:
		board.makeStone(adjustPos, turn, &scene);
		// ^[ւ
		turn = Stone::opposite(turn);
		// ^[ւĂ烁bZ[W\
		putMessage(fk_StrPrintf("(%d, %d)ɒu܂B", pos.x, pos.y));
		// ̎Ԃ̐lu邩ǂ`FbN
		if(board.isPuttable(turn) == false) {
			if(board.isPuttable(Stone::opposite(turn)) == false) {
				// [7/3]s
				int bNum, wNum;
				StoneColor result = board.judge(&bNum, &wNum);
				turn = ST_DEFAULT;
				if(result == ST_BLACK) {
					putMessage(fk_StrPrintf("%d  %d ŁȀłB", bNum, wNum));
				} else if(result == ST_WHITE) {
					putMessage(fk_StrPrintf("%d  %d ŁȀłB", bNum, wNum));
				} else {
					putMessage(fk_StrPrintf("%d  %d ŁAłB", bNum, wNum));
				}
			} else {
				turn = Stone::opposite(turn);
				putMessage("΂Ď̃^[łB");
			}
		}
		break;
	case CR_OUT_RANGE:
		putMessage("Ֆʂ̊Oɂ͒u܂B");
		break;
	case CR_FAILED:
		putMessage(fk_StrPrintf("(%d, %d)͒u܂B", pos.x, pos.y));
		break;
	default:
		putMessage("ɒuĂ܂B");
		break;
	}

	return;
}

// [6/5]u΂̃f[^eLXgt@Cɏo
bool OthelloApp::saveState(void)
{
	ofstream	out_file("savedata.txt");

	if(!out_file.is_open()) return false;

	// oBoardNXɔC
	board.saveState(out_file);

	// ̃^[ŏo
	out_file << turn << endl;

	out_file.close();

	return true;
}

// [6/5]eLXgt@C͂Au΂̏Ԃ𕜌
bool OthelloApp::loadState(void)
{
	ifstream		in_file("savedata.txt");
	string			str;

	// ǂݍݏBoardNXɔC
	if(!in_file.is_open()) return false;

	// ŏ̍s[BOARD INFO]ȂG[
	if(getline(in_file, str)) {
		if(str != "[BOARD INFO]") return false;
	} else {
		return false;
	}

	bool result = board.loadState(in_file, &scene);

	// ̎ԂǂݍŔf
	if(getline(in_file, str)) {
		int val = atoi(str.c_str());
		if(ST_BLACK == val) {
			turn = ST_BLACK;
		} else if(ST_WHITE == val) {
			turn = ST_WHITE;
		} else {
			result = false;
		}
	} else {
		result = false;
	}

	in_file.close();

	return result;
}

void OthelloApp::putMessage(string argText)
{
	textSpr.drawText(argText, true);
	if(turn == ST_BLACK) {
		textSpr.drawText("\n̎ԂłB", false);
	} else if(turn == ST_WHITE) {
		textSpr.drawText("\n̎ԂłB", false);
	}
	textSpr.setPositionLT(-WINDOW_WIDTH*0.5, WINDOW_HEIGHT*0.5);

	return;
}
